@@ -295,6 +295,22 @@ class McritClient:
         response = requests.get(f"{self.mcrit_server}/matches/sample/{sample_id}/{other_sample_id}",params=params)
         return handle_response(response)
 
+    def requestMatchesCross(
+        self,
+        sample_ids,
+        minhash_threshold=None,
+        pichash_size=None,
+        force_recalculation=False,
+    ) -> None:
+        params = self._getMatchingRequestParams(
+            minhash_threshold, pichash_size, force_recalculation
+        )
+        response = requests.get(
+            f"{self.mcrit_server}/matches/sample/cross/{','.join([str(id) for id in sample_ids])}",
+            params=params
+        )
+        return handle_response(response)
+
     def getMatchesForPicHash(self, pichash, summary=False):
         summary_string = "/summary" if summary else ""
         response = requests.get(f"{self.mcrit_server}/query/pichash/{pichash:016x}{summary_string}")
@@ -411,6 +427,27 @@ class McritClient:
         response = requests.post(f"{self.mcrit_server}/import", json=import_data)
         return handle_response(response)
 
+    ###########################################
+    ### Import / Export
+    ###########################################
+
+    def getUniqueBlocksForSamples(self, sample_ids: List[int]) -> Dict:
+        if isinstance(sample_ids, list) and all(isinstance(item, int) for item in sample_ids):
+            sample_ids_as_str = ",".join([str(sample_id) for sample_id in sample_ids])
+            response = requests.get(f"{self.mcrit_server}/uniqueblocks/samples/{sample_ids_as_str}")
+            result_data = handle_response(response)
+        else:
+            raise ValueError("sample_ids must be a list of int.")
+        return result_data
+
+    def getUniqueBlocksForFamily(self, family_id: int) -> Dict:
+        if isinstance(family_id, int):
+            response = requests.get(f"{self.mcrit_server}/uniqueblocks/family/{family_id}")
+            result_data = handle_response(response)
+        else:
+            raise ValueError("sample_ids must be a list of int.")
+        return result_data
+
     ###########################################
     ### Search
     ###########################################
