@@ -3,9 +3,11 @@ import re
 import json
 import time
 import logging
-from typing import Dict
+from typing import Dict, List
 
 from smda.common.SmdaReport import SmdaReport
+from smda.common.SmdaInstruction import SmdaInstruction
+from smda.intel.IntelInstructionEscaper import IntelInstructionEscaper
 
 from mcrit.config.McritConfig import McritConfig
 from mcrit.config.MinHashConfig import MinHashConfig
@@ -258,6 +260,13 @@ class MinHashIndex(QueueRemoteCaller(Worker)):
             report_json = json.load(fin)
         report = SmdaReport.fromDict(report_json)
         return self.addReport(report, calculate_hashes=calculate_hashes, calculate_matches=calculate_matches)
+    
+    def getMatchesCross(self, sample_ids:List[int], force_recalculation=False, **params):
+        sample_to_job_id = {}
+        for id in sample_ids:
+            job_id = self.getMatchesForSample(id, force_recalculation=force_recalculation, **params)
+            sample_to_job_id[id] = job_id
+        return self.combineMatchesToCross(sample_to_job_id, await_jobs=[*sample_to_job_id.values()], force_recalculation=force_recalculation)
 
     #### SIMPLE LOOKUPS ####
     def getFamily(self, family_id):
@@ -318,7 +327,7 @@ class MinHashIndex(QueueRemoteCaller(Worker)):
             "status": {
                 "db_state": storage_stats["db_state"],
                 "storage_type": self._storage_config.STORAGE_METHOD,
-                "storage_bands": len(storage_stats["bands"]),
+                "num_bands": storage_stats["num_bands"],
                 "num_samples": storage_stats["num_samples"],
                 "num_families": storage_stats["num_families"],
                 "num_functions": storage_stats["num_functions"],
@@ -336,6 +345,22 @@ class MinHashIndex(QueueRemoteCaller(Worker)):
     def getSampleBySha256(self, sample_sha256): 
         return self._storage.getSampleBySha256(sample_sha256)
 
+    def getUniqueBlocks(self, sample_ids):
+        unique_blocks = self._storage.getUniqueBlocks(sample_ids)
+        # enrich with escaped sequences
+        sample_addr_borders = {}
+        for sample_id in sample_ids:
+            sample_entry = self._storage.getSampleById(sample_id)
+            sample_addr_borders[sample_id] = {"lower": sample_entry.base_addr, "upper": sample_entry.base_addr + sample_entry.binary_size}
+        for block_hash, entry in unique_blocks.items():
+            sample_id = entry["sample_id"]
+            escaped_sequences = []
+            for instruction in entry["instructions"]:
+                smda_instruction = SmdaInstruction(instruction)
+                escaped_sequences.append(smda_instruction.getEscapedBinary(IntelInstructionEscaper, escape_intraprocedural_jumps=True, lower_addr=sample_addr_borders[sample_id]["lower"], upper_addr=sample_addr_borders[sample_id]["upper"]))
+            unique_blocks[block_hash]["escaped_sequence"] = " ".join(escaped_sequences)
+        return unique_blocks
+
     #### SEARCH ####
 
     # When performing an initial search, the cursor should be set to None.
