@@ -5,6 +5,7 @@ import logging
 import datetime
 import traceback
 from operator import itemgetter
+from itertools import zip_longest
 from typing import Any, TYPE_CHECKING, Dict, Iterable, List, Optional, Set, Tuple, Union
 
 LOGGER = logging.getLogger(__name__)
@@ -638,20 +639,22 @@ class MongoDbStorage(StorageInterface):
                 candidates.update(band_document["function_ids"])
         return candidates
 
-    # TODO return type
-    def _getCacheDataForFunctionIds(self, function_ids: List[int]):
+    def _getCacheDataForFunctionIds(self, function_ids: List[int]) -> Dict:
         cache_data = {}
         sample_ids = {}
-        minhashs = {}
-        # TODO why is list necessary here?
-        for function_document in self._database.functions.find(
-            {"function_id": {"$in": list(function_ids)}}, {"_id": 0}
-        ):
-            self._decodeFunction(function_document)
-            entry = FunctionEntry.fromDict(function_document)
-            minhashs[entry.function_id] = entry.minhash
-            sample_ids[entry.function_id] = entry.sample_id
-        cache_data["func_id_to_minhash"] = minhashs
+        minhashes = {}
+        # process this in batches as the number of function_ids can be exceedingly large, pushing beyond Mongo's 16M limit
+        for sliced_ids in zip_longest(*[iter(function_ids)]*500000):
+            query_function_ids = [fid for fid in sliced_ids if fid is not None]
+            for function_document in self._database.functions.find(
+                {"function_id": {"$in": list(query_function_ids)}}, {"_id": 0, "sample_id": 1, "minhash": 1, "function_id": 1}
+            ):
+                function_id = function_document["function_id"]
+                sample_id = function_document["sample_id"]
+                minhash = bytes.fromhex(function_document["minhash"])
+                minhashes[function_id] = minhash
+                sample_ids[function_id] = sample_id
+        cache_data["func_id_to_minhash"] = minhashes
         cache_data["func_id_to_sample_id"] = sample_ids
         return cache_data
 
@@ -775,9 +778,6 @@ class MongoDbStorage(StorageInterface):
 
     # TODO add types
     def getStats(self) -> Dict:
-        band_info = {}
-        for band_id in range(self._config.STORAGE_NUM_BANDS):
-            band_info[band_id] = self._database["band_%d" % band_id].count_documents(filter={})
         # we will have to work around using .aggregate(), as a collection with unique pic hashes will easily exceed 16M
         # https://stackoverflow.com/questions/20348093/mongodb-aggregation-how-to-get-total-records-count
         num_unique_pichashes = 0
@@ -788,9 +788,8 @@ class MongoDbStorage(StorageInterface):
             "num_families": self._database.families.count_documents(filter={}),
             "num_samples": self._database.samples.count_documents(filter={}),
             "num_functions": self._database.functions.count_documents(filter={}),
-            "bands": band_info,
+            "num_bands": self._config.STORAGE_NUM_BANDS,
             "num_pichashes": num_unique_pichashes,
-            "num_matches": self._database.matches.count_documents(filter={}),
         }
         return stats
 
@@ -805,6 +804,47 @@ class MongoDbStorage(StorageInterface):
                 unhashed_functions.append(FunctionEntry.fromDict(function_document))
         return unhashed_functions
 
+    def getUniqueBlocks(self, sample_ids: Optional[List[int]] = None) -> Dict:
+        # query once to get all blocks from the functions of our samples
+        candidate_picblockhashes = {}
+        for entry in self._database.functions.find({"sample_id": {"$in": sample_ids}, "_picblockhashes": {"$exists": True, "$ne": [] }}, {"function_id": 1, "sample_id": 1, "_picblockhashes": 1, "_id": 0}):
+            sample_id = entry["sample_id"]
+            for block_entry in entry["_picblockhashes"]:
+                block_hash = block_entry["hash"]
+                if block_hash not in candidate_picblockhashes:
+                    candidate_picblockhashes[block_hash] = {
+                        "samples": set(),
+                        "length": block_entry["length"],
+                        "function_id": entry["function_id"],
+                        "sample_id": sample_id,
+                        "offset": block_entry["offset"],
+                        "instructions": [],
+                        "escaped_sequence": ""
+                    }
+                candidate_picblockhashes[block_hash]["samples"].add(sample_id)
+        LOGGER.info(f"Found {len(candidate_picblockhashes)} candidate picblock hashes")
+        # remove those that are not unique
+        for entry in self._database.functions.find({"_picblockhashes": {"$exists": True, "$ne": [] }}, {"sample_id": 1, "_picblockhashes": 1, "_id": 0}):
+            sample_id = entry["sample_id"]
+            if sample_id not in sample_ids:
+                for block_entry in entry["_picblockhashes"]:
+                    candidate_picblockhashes.pop(block_entry["hash"], None)
+        LOGGER.info(f"Reduced to {len(candidate_picblockhashes)} unique picblock hashes")
+        # iterate over candidates by function_id and extract instructions
+        function_id_to_block_offsets = {}
+        for picblockhash, entry in candidate_picblockhashes.items():
+            candidate_picblockhashes[picblockhash]["samples"] = sorted(list(entry["samples"]))
+            if entry["function_id"] not in function_id_to_block_offsets:
+                function_id_to_block_offsets[entry["function_id"]] = []
+            function_id_to_block_offsets[entry["function_id"]].append((entry["offset"], picblockhash))
+        for entry in self._database.functions.find({"function_id": {"$in": list(function_id_to_block_offsets.keys())}}, {"function_id": 1, "_xcfg": 1, "_id": 0}):
+            function_id = entry["function_id"]
+            self._decodeXcfg(entry)
+            for block_offset, picblockhash in function_id_to_block_offsets[function_id]:
+                candidate_picblockhashes[picblockhash]["instructions"] = entry["xcfg"]["blocks"][str(block_offset)]
+        LOGGER.info(f"Instructions for {len(candidate_picblockhashes)} blocks extracted.")
+        return candidate_picblockhashes
+
 
     ##### helpers for search ######
 
